use serde::Serializer;
use uuid::Uuid;
use chrono::{DateTime, Utc};

pub fn to_simple_uuid<S>(uuid: &Uuid, serializer: S) -> Result<S::Ok, S::Error>
    where S: Serializer
{
    serializer.serialize_str(&uuid.to_simple().encode_lower(&mut [0; 32]))
}

pub fn to_date_time<S>(dt: &DateTime<Utc>, serializer: S) -> Result<S::Ok, S::Error>
    where S: Serializer
{
    serializer.serialize_str(&dt.format("%Y-%m-%d %H:%M:%S").to_string()[..])
}

pub fn to_hsl<S>(n: &u32, serializer: S) -> Result<S::Ok, S::Error>
    where S: Serializer
{
    let [h1, h2, s, l] = n.to_be_bytes();
    let h = u16::from_be_bytes([h1, h2]);

    serializer.serialize_str(&format!("hsl({},{}%,{}%)", h % 360, s % 101, l % 101)[..])
}
