use handlebars::Handlebars;
use std::cmp::min;
use chrono::NaiveDate;

pub fn init_templates<'a>() -> Result<Handlebars<'a>, Box<dyn std::error::Error>> {
    let mut hb = Handlebars::new();

    hb.register_templates_directory(".hbs", "templates/")?;

    hb.register_helper("first5", Box::new(first5_helper));
    hb.register_helper("skip5", Box::new(skip5_helper));
    hb.register_helper("sub1", Box::new(sub1_helper));
    hb.register_helper("isPrevDay", Box::new(is_prev_day_helper));
    hb.register_helper("toDaysSeparatorDate", Box::new(to_days_separator_date_helper));

    Ok(hb)
}

handlebars_helper!(first5_helper: |s: str| &s[0..min(5, s.len())]);
handlebars_helper!(skip5_helper: |s: str| &s[5..s.len()]);
handlebars_helper!(sub1_helper: |x: i64| x - 1);
handlebars_helper!(is_prev_day_helper: |date1: str, date2: str|
    // e.g. "2020-10-16 11:38:03"
    &date1[0..min(10, date1.len())] < &date2[0..min(10, date1.len())]
);
handlebars_helper!(to_days_separator_date_helper: |date: str|
    NaiveDate::parse_from_str(&date[0..min(10, date.len())], "%Y-%m-%d")
        .map(|date: NaiveDate| date.format("%-e %B %Y").to_string())
        .unwrap_or("".to_string())
);

