from requests.cookies import cookiejar_from_dict
from shared import *


def authorize(data, referrer=None, params=None, clear_cookies=True, cookies=None):
    if clear_cookies:
        session.cookies.clear()

    if cookies:
        session.cookies = cookiejar_from_dict(cookies)

    headers = {'Referer': referrer} if referrer else None

    r = session.post(f"{base_url}/authorize", data=data, params=params,
                     headers=headers, allow_redirects=False, cookies=cookies)
    print(f"POST {r.url}\n{data}")
    return r


def test_authorize_success(nickname=valid_nickname, password=valid_password):
    # params are ignored, additional form data is ignored, referrer is not required
    r = authorize(data={
        'nickname': nickname,
        'password': password,
        'backUrl': f'{base_path}/boards',
        'some': 'thing',
    }, cookies={'clientId': client_id}, params={'p': '12'})
    assert_found(r, f"{base_path}/boards")
    return r


def test_authorize_errors():
    # nickname required
    r = authorize(data={'password': valid_nickname})
    assert r.text == 'Request body deserialize error: missing field `nickname`'
    assert r.status_code == 400

    # password required
    r = authorize(data={'nickname': valid_nickname})
    assert r.text == 'Request body deserialize error: missing field `password`'
    assert r.status_code == 400

    # nickname invalid
    for nickname in invalid_nicknames:
        r = authorize(data={'nickname': nickname, 'password': 'b'})
        assert r.status_code == 200
        assert 'Invalid nickname' in r.text

    # anon cannot log in
    r = authorize(data={'nickname': 'Anonymous', 'password': 'uOGghquQ5smwXtsxqfeq'})
    assert r.status_code == 200
    assert 'Anonymous cannot log in' in r.text

    # password length limit
    r = authorize(data={'nickname': valid_nickname, 'password': 'random' * 30})
    assert r.status_code == 200
    assert 'Password length is over limit' in r.text

    # user banned
    r = authorize(data={'nickname': 'banned', 'password': 'b'})
    assert r.status_code == 200
    assert 'The user is banned' in r.text

    # incorrect credentials
    for (nickname, password) in [('notexist', 'asdf'), ('someone', 'asdf')]:
        r = authorize(data={'nickname': nickname, 'password': password})
        assert r.status_code == 200
        assert 'Incorrect nickname or password' in r.text

    # max length
    r = authorize(data={'something': 'a' * 256})
    assert r.status_code == 413

    r = session.get(f"{base_url}/authorize")
    print(f"GET {r.url}")
    assert r.status_code == 405
