from shared import *
from itertools import product
import json


def import_ui_state(ui_state, back_url=None, params=None, clear_cookies=True):
    if clear_cookies:
        session.cookies.clear()

    files = {
        "uiState": ("pfpmd_ui3_settings.json", json.dumps(ui_state), 'application/json'),
    }
    if back_url is not None:
        files["backUrl"] = (None, back_url)

    r = session.post(f"{base_url}/importUiState", params=params,
                     files=files, allow_redirects=False)
    print(f"POST {r.url}\n{files}")
    return r


def test_import_ui_state():
    redirect_url1 = f"{base_path}/help"
    redirect_url2 = f"{base_path}/boards"

    for settings in product(
            (None, client_id),
            (False, True), (False, True),
            (900, 1000), (False, True), (False, True),
            (None, redirect_url1, redirect_url2)
    ):
        _client_id, fix_menu, show_thumbs, max_width, compact, only_chat_posts, back_url = settings
        cookies = {}
        if _client_id is not None:
            cookies['clientId'] = _client_id
        cookies['fixMenu'] = fix_menu
        cookies['showThumbs'] = show_thumbs
        cookies['maxWidth'] = max_width
        cookies['compact'] = compact
        cookies['onlyChatPosts'] = only_chat_posts

        ui_state = {
            "ui": "ui3",
            "version": "0.1.4",
            "cookies": cookies
        }

        # params are ignored
        r = import_ui_state(ui_state, back_url, params={'p': '12'})
        assert_found(r, back_url if back_url is not None else f"{base_path}/")
        if _client_id is None:
            assert r.cookies.get("clientId") is None
        else:
            assert r.cookies.get("clientId") == _client_id
        assert r.cookies.get("fixMenu") == json.dumps(fix_menu)
        assert r.cookies.get("showThumbs") == json.dumps(show_thumbs)
        assert r.cookies.get("maxWidth") == json.dumps(max_width)
        assert r.cookies.get("compact") == json.dumps(compact)
        assert r.cookies.get("onlyChatPosts") == json.dumps(only_chat_posts)


def test_import_ui_state_errors():
    # wrong ui value
    r = import_ui_state(ui_state={
        "ui": "asdf",
        "version": "0.1.4",
        "cookies": {
            "fixMenu": False,
            "showThumbs": False,
            "maxWidth": 900,
            "compact": False,
            "onlyChatPosts": False,
        }
    })
    assert "Invalid ui" in r.text
    assert r.status_code == 500

    # required fields
    r = import_ui_state(ui_state={
        "ui": "ui3",
        "version": "0.1.4",
        "cookies": {
            # "fixMenu": False,
            "showThumbs": False,
            "maxWidth": 900,
            "compact": False,
            "onlyChatPosts": False,
        }
    })
    assert 'missing field `fixMenu`' in r.text
    assert r.status_code == 500

    r = import_ui_state(ui_state={
        "ui": "ui3",
        "version": "0.1.4",
        "cookies": {
            "fixMenu": False,
            # "showThumbs": False,
            "maxWidth": 900,
            "compact": False,
            "onlyChatPosts": False,
        }
    })
    assert 'missing field `showThumbs`' in r.text
    assert r.status_code == 500

    r = import_ui_state(ui_state={
        "ui": "ui3",
        "version": "0.1.4",
        "cookies": {
            "fixMenu": False,
            "showThumbs": False,
            # "maxWidth": 900,
            "compact": False,
            "onlyChatPosts": False,
        }
    })
    assert 'missing field `maxWidth`' in r.text
    assert r.status_code == 500

    r = import_ui_state(ui_state={
        "ui": "ui3",
        "version": "0.1.4",
        "cookies": {
            "fixMenu": False,
            "showThumbs": False,
            "maxWidth": 900,
            # "compact": False,
            "onlyChatPosts": False,
        }
    })
    assert 'missing field `compact`' in r.text
    assert r.status_code == 500

    r = import_ui_state(ui_state={
        "ui": "ui3",
        "version": "0.1.4",
        "cookies": {
            "fixMenu": False,
            "showThumbs": False,
            "maxWidth": 900,
            "compact": False,
            # "onlyChatPosts": False,
        }
    })
    assert 'missing field `onlyChatPosts`' in r.text
    assert r.status_code == 500

    # max_width invalid uints
    for max_width in invalid_uints:
        r = import_ui_state(ui_state={
            "ui": "ui3",
            "version": "0.1.4",
            "cookies": {
                "fixMenu": False,
                "showThumbs": False,
                "maxWidth": max_width,
                "compact": False,
                "onlyChatPosts": False,
            }
        })
        assert 'Failed to parse uiState: invalid' in r.text
        assert r.status_code == 500

    # check one of the fields for invalid booleans
    for fix_menu in invalid_booleans:
        r = import_ui_state(ui_state={
            "ui": "ui3",
            "version": "0.1.4",
            "cookies": {
                "fixMenu": fix_menu,
                "showThumbs": False,
                "maxWidth": 900,
                "compact": False,
                "onlyChatPosts": False,
            }
        })
        assert 'Failed to parse uiState: invalid' in r.text
        assert r.status_code == 500

    r = import_ui_state(ui_state={'asdf': 123})
    assert 'Failed to parse uiState' in r.text
    assert r.status_code == 500

    # max length
    r = import_ui_state(ui_state={}, back_url="ab" * 1024)
    assert r.status_code == 413

    r = session.get(f"{base_url}/importUiState")
    print(f"GET {r.url}")
    assert r.status_code == 405
