from requests.cookies import cookiejar_from_dict
from shared import *
from test_authorize import test_authorize_success


def logout(data, referrer=None, params=None, clear_cookies=True, cookies=None):
    if clear_cookies:
        session.cookies.clear()

    if cookies:
        session.cookies = cookiejar_from_dict(cookies)

    headers = {'Referer': referrer} if referrer else None

    r = session.post(f"{base_url}/logout", data=data, params=params,
                     headers=headers, allow_redirects=False, cookies=cookies)
    print(f"POST {r.url}\n{data}")
    return r


def test_logout_success():
    access_token = test_authorize_success().cookies.get('accessToken')

    # params are ignored, additional form data is ignored, referrer is not required
    r = logout(data={
        'backUrl': f'{base_path}/boards', # not used, we get redirected to /
        'some': 'thing',
    }, cookies={'clientId': client_id, 'accessToken': access_token}, params={'p': '12'})
    assert_found(r, f'{base_path}/')
    assert r.cookies.get('accessToken') is None

    # doesn't matter if unauthorized. The request just removes the accessToken cookie
    r = logout(data={'some': 'thing'}, cookies={'clientId': client_id}, params={'p': '12'})
    assert_found(r, f'{base_path}/')
    assert r.cookies.get('accessToken') is None


def test_logout_errors():
    # max length
    r = logout(data={
        'backUrl': 'a' * 1024
    }, cookies={'clientId': client_id})
    assert r.status_code == 413

    r = session.get(f"{base_url}/logout")
    print(f"GET {r.url}")
    assert r.status_code == 405
