from shared import *
from itertools import product


def save_settings(data, params=None, clear_cookies=True):
    if clear_cookies:
        session.cookies.clear()

    r = session.post(f"{base_url}/saveSettings", data=data, params=params,
                     allow_redirects=False)
    print(f"POST {r.url}\n{data}")
    return r


def test_save_settings():
    redirect_url1 = f"{base_path}/help"
    redirect_url2 = f"{base_path}/boards"

    for settings in product((None, 'false', 'true'), (None, 'false', 'true'), (None, 'false', 'true'),
                            ('900', '1000'), (None, 'false', 'true'), (None, 'false', 'true'),
                            (None, redirect_url1, redirect_url2)):
        change_colors, fix_menu, show_thumbs, max_width, compact_theme, only_chat_posts, back_url = settings
        data = {}
        if change_colors is not None:
            data['changeColors'] = change_colors
        if fix_menu is not None:
            data['fixMenu'] = fix_menu
        if show_thumbs is not None:
            data['showThumbs'] = show_thumbs
        if max_width is not None:
            data['maxWidth'] = max_width
        if compact_theme is not None:
            data['compactTheme'] = compact_theme
        if only_chat_posts is not None:
            data['onlyChatPosts'] = only_chat_posts
        if back_url is not None:
            data['backUrl'] = back_url

        # params are ignored
        r = save_settings(data=data, params={'p': '12'})
        assert_found(r, back_url if back_url is not None else f"{base_path}/")
        assert r.cookies.get("clientId")
        assert r.cookies.get("fixMenu") == 'false' if fix_menu is None else fix_menu
        assert r.cookies.get("showThumbs") == 'false' if show_thumbs is None else show_thumbs
        assert r.cookies.get("maxWidth") == '900' if max_width is None else max_width
        assert r.cookies.get("compact") == 'false' if compact_theme is None else compact_theme
        assert r.cookies.get("onlyChatPosts") == 'false' if only_chat_posts is None else only_chat_posts


def test_save_settings_errors():
    # max_width required
    r = save_settings(data={'compact': 'true'})
    assert r.text == 'Request body deserialize error: missing field `maxWidth`'
    assert r.status_code == 400

    # max length
    r = save_settings(data={'something': 'a' * 1024})
    assert r.status_code == 413

    r = session.get(f"{base_url}/saveSettings")
    print(f"GET {r.url}")
    assert r.status_code == 405
