from requests.cookies import cookiejar_from_dict
from shared import *
import subprocess


def signup(data, referrer=None, params=None, clear_cookies=True, cookies=None):
    if clear_cookies:
        session.cookies.clear()

    if cookies:
        session.cookies = cookiejar_from_dict(cookies)

    headers = {'Referer': referrer} if referrer else None

    r = session.post(f"{base_url}/signup", data=data, params=params,
                     headers=headers, allow_redirects=False, cookies=cookies)
    print(f"POST {r.url}\n{data}")
    return r


def get_pow(nickname):
    cmd = f'hashcash -ub20 -x pfpmd -m {nickname}'
    print(f"Calculating PoW: {cmd}")
    pow = subprocess.getoutput(cmd).strip()
    print(pow)
    return pow


def test_signup_success():
    # params are ignored, additional form data is ignored, referrer is not required
    r = signup(data={
        'nickname': valid_nickname,
        'password': valid_password,
        'pow': get_pow(valid_nickname),
        'backUrl': f'{base_path}/boards',
        'some': 'thing',
    }, cookies={'clientId': client_id}, params={'p': '12'})
    assert r.status_code == 200
    assert 'User created. Now you can log in.' in r.text


def test_signup_errors():
    # nickname required
    r = signup(data={'password': valid_password})
    assert r.text == 'Request body deserialize error: missing field `nickname`'
    assert r.status_code == 400

    # password required
    r = signup(data={'nickname': valid_nickname})
    assert r.text == 'Request body deserialize error: missing field `password`'
    assert r.status_code == 400

    # pow required
    r = signup(data={'nickname': valid_nickname, 'password': valid_password})
    assert r.text == 'Request body deserialize error: missing field `pow`'
    assert r.status_code == 400

    # nickname invalid
    for nickname in invalid_nicknames:
        r = signup(data={'nickname': nickname, 'password': 'b', 'pow': 'c'})
        assert r.status_code == 200
        assert 'Invalid nickname' in r.text

    # password length limit
    r = signup(data={'nickname': valid_nickname, 'password': 'random' * 30, 'pow': 'c'})
    assert r.status_code == 200
    assert 'Password length is over limit' in r.text

    # pow length limit
    r = signup(data={'nickname': valid_nickname, 'password': valid_password, 'pow': 'random' * 30})
    assert r.status_code == 200
    assert 'PoW length is over limit' in r.text

    # other things are already checked in api tests

    # max length
    r = signup(data={'something': 'a' * 1024})
    assert r.status_code == 413

    r = session.get(f"{base_url}/signup")
    print(f"GET {r.url}")
    assert r.status_code == 405
