from requests.cookies import cookiejar_from_dict
from shared import *
from test_authorize import test_authorize_success


def update_hidden_by_op(data, referrer=None, params=None, clear_cookies=True, cookies=None):
    if clear_cookies:
        session.cookies.clear()

    if cookies:
        session.cookies = cookiejar_from_dict(cookies)

    headers = {'Referer': referrer} if referrer else None

    r = session.post(f"{base_url}/updateHiddenByOp", data=data, params=params,
                     headers=headers, allow_redirects=False, cookies=cookies)
    print(f"POST {r.url}\n{data}")
    return r


test_post_id = '50e4f41e-9e42-46e7-a048-b512923271d9'
valid_client_id = 'tv14zg9ika'


def test_update_hidden_by_op():
    access_token = test_authorize_success().cookies.get('accessToken')

    for hidden_by_op in ('true', 'false'):
        # referrer is not required
        r = update_hidden_by_op(data={
            'postId': test_post_id,
            'hiddenByOp': hidden_by_op,
            'backUrl': f'{base_path}/help'
        }, cookies={'clientId': valid_client_id, 'accessToken': access_token})
        assert_found(r, f"{base_path}/help")

    # code is not required code we should get 302
    r = update_hidden_by_op(data={
        'postId': test_post_id,
        'hiddenByOp': 'true',
        'backUrl': f'{base_path}/help'
    }, cookies={'clientId': valid_client_id, 'accessToken': access_token})
    assert_found(r, f"{base_path}/help")

    # params are ignored
    r = update_hidden_by_op(data={
        'postId': test_post_id,
        'hiddenByOp': 'true',
        'backUrl': f'{base_path}/help'
    }, cookies={'clientId': valid_client_id, 'accessToken': access_token}, params={'p': '12'})
    assert_found(r, f"{base_path}/help")

    # additional form data is ignored
    r = update_hidden_by_op(data={
        'postId': test_post_id,
        'hiddenByOp': 'true',
        'backUrl': f'{base_path}/help',
        'some': 'thing',
    }, cookies={'clientId': valid_client_id, 'accessToken': access_token}, params={'p': '12'})
    assert_found(r, f"{base_path}/help")

    # # commented out because it works only if anonymous posting is allowed
    # # this clientId is invalid for that thread
    # r = update_hidden_by_op(data={
    #     'postId': test_post_id,
    #     'hiddenByOp': 'true',
    #     'backUrl': f'{base_path}/help'
    # }, cookies={'clientId': client_id, 'accessToken': access_token})
    # assert_success_response(r)
    # assert 'invalid clientId' in r.text

    # if user is logged in, doesn't matter if clientId is invalid for that thread
    r = update_hidden_by_op(data={
        'postId': test_post_id,
        'hiddenByOp': 'true',
        'backUrl': f'{base_path}/help'
    }, cookies={'clientId': client_id, 'accessToken': access_token})
    assert_found(r, f"{base_path}/help")

    r = update_hidden_by_op(data={
        'postId': test_post_id,
        'hiddenByOp': 'true',
        'backUrl': f'{base_path}/help'
    }, cookies={'accessToken': access_token})
    assert_success_response(r)
    assert 'Invalid clientId' in r.text

    r = update_hidden_by_op(data={
        'postId': test_post_id,
        'hiddenByOp': 'true',
        'backUrl': f'{base_path}/help'
    }, cookies={'clientId': client_id,
                'accessToken': test_authorize_success('asdf', 'asdf').cookies.get('accessToken')})
    assert_success_response(r)
    assert 'Invalid nickname' in r.text


def test_update_hidden_by_op_errors():
    for hidden_by_op in invalid_booleans:
        r = update_hidden_by_op(data={
            'postId': test_post_id,
            'hiddenByOp': hidden_by_op,
            'backUrl': f'{base_path}/help'
        }, cookies={'clientId': valid_client_id})
        assert r.status_code == 400

    for post_id in invalid_uuids:
        r = update_hidden_by_op(data={
            'postId': post_id,
            'hiddenByOp': 'true',
            'backUrl': f'{base_path}/help'
        }, cookies={'clientId': valid_client_id})
        assert r.status_code == 400

    # backUrl is required
    r = update_hidden_by_op(data={
        'postId': test_post_id,
        'hiddenByOp': 'true'
    }, cookies={'clientId': valid_client_id})
    assert r.status_code == 400

    # hiddenByOp is required
    r = update_hidden_by_op(data={
        'postId': test_post_id,
        'backUrl': f'{base_path}/help'
    }, cookies={'clientId': valid_client_id})
    assert r.status_code == 400

    # postId is required
    r = update_hidden_by_op(data={
        'hiddenByOp': 'true',
        'backUrl': f'{base_path}/help'
    }, cookies={'clientId': valid_client_id})
    assert r.status_code == 400

    # unauthorized
    r = update_hidden_by_op(data={
        'postId': chat_thread_id,
        'hiddenByOp': 'true',
        'backUrl': f'{base_path}/help'
    }, cookies={'clientId': valid_client_id})
    assert_unauthorized(r)

    # post not found
    r = update_hidden_by_op(data={
        'postId': chat_thread_id,
        'hiddenByOp': 'true',
        'backUrl': f'{base_path}/help'
    }, cookies={'clientId': valid_client_id,
                'accessToken': test_authorize_success().cookies.get('accessToken')})
    assert_not_found(r)

    # max length
    r = update_hidden_by_op(data={
        'postId': test_post_id,
        'hiddenByOp': 'true',
        'backUrl': 'a' * 1024
    }, cookies={'clientId': client_id})
    assert r.status_code == 413

    r = session.get(f"{base_url}/updateHiddenByOp")
    print(f"GET {r.url}")
    assert r.status_code == 405
